"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useTts = void 0;
const react_1 = require("react");
const controller_js_1 = require("./controller.cjs");
const utils_js_1 = require("./utils.cjs");
const highlighter_js_1 = require("./highlighter.cjs");
const isObjectReactNode = (value) => typeof value === 'object' && value !== null;
const parseChildrenRecursively = ({ children, buffer, boundary, markColor, markBackgroundColor, markTextAsSpoken }) => {
    return react_1.Children.map(children, (child) => {
        let currentChild = child;
        if ((0, react_1.isValidElement)(child)) {
            const childProps = isObjectReactNode(child.props) ? child.props : {};
            currentChild = (0, react_1.cloneElement)(child, Object.assign(Object.assign({}, childProps), { 
                // @ts-expect-error - `children` is not a valid prop for ReactElement
                children: parseChildrenRecursively({
                    buffer,
                    boundary,
                    markColor,
                    markBackgroundColor,
                    markTextAsSpoken,
                    children: childProps.children
                }) }));
        }
        if ((0, utils_js_1.isStringOrNumber)(child)) {
            const text = child.toString();
            const { word, startChar, endChar } = boundary;
            const bufferTextLength = buffer.text.length;
            buffer.text += `${text} `;
            if (markTextAsSpoken && word) {
                const start = startChar - bufferTextLength;
                const end = endChar - bufferTextLength;
                const prev = text.substring(0, start);
                const found = text.substring(start, end);
                const after = text.substring(end, text.length);
                if (found) {
                    const Highlight = (0, react_1.createElement)(highlighter_js_1.Highlighter, {
                        text: found,
                        mark: (0, utils_js_1.stripPunctuation)(found),
                        color: markColor,
                        backgroundColor: markBackgroundColor
                    });
                    const Highlighted = (0, react_1.createElement)(react_1.Fragment, { key: `tts-${start}-${end}` }, prev, Highlight, after);
                    return Highlighted;
                }
            }
        }
        return currentChild;
    });
};
const defaultBoundary = { word: '', startChar: 0, endChar: 0 };
const reducer = (state, action) => {
    switch (action.type) {
        case 'pause':
            return Object.assign(Object.assign({}, state), { isPlaying: false, isPaused: true, isError: false });
        case 'play':
        case 'reset':
            return Object.assign(Object.assign({}, state), { isPlaying: true, isPaused: false, isError: false, boundary: defaultBoundary });
        case 'end':
            return Object.assign(Object.assign({}, state), { isPlaying: false, isPaused: false, isError: false, boundary: defaultBoundary });
        case 'error':
            return Object.assign(Object.assign({}, state), { isPlaying: false, isPaused: false, isError: true, boundary: defaultBoundary });
        case 'ready':
            return Object.assign(Object.assign({}, state), { isReady: true });
        case 'boundary':
            return Object.assign(Object.assign({}, state), { boundary: Object.assign(Object.assign({}, state.boundary), action.payload) });
        case 'voices':
            return Object.assign(Object.assign({}, state), { voices: action.payload });
        case 'stop':
            return Object.assign(Object.assign({}, state), { isPlaying: false, isPaused: false, isError: false });
        case 'muted':
            return Object.assign(Object.assign({}, state), { isMuted: true });
        case 'unmuted':
            return Object.assign(Object.assign({}, state), { isMuted: false });
    }
};
const useTts = ({ lang, rate, volume, voice, children, markColor, markBackgroundColor, onStart, onPause, onBoundary, onEnd, onError, onVolumeChange, onPitchChange, onRateChange, onNotSupported, fetchAudioData, autoPlay = false, markTextAsSpoken = false }) => {
    const spokenTextRef = (0, react_1.useRef)('');
    const handleNotSupported = onNotSupported || utils_js_1.noop;
    const isSynthSupported = typeof globalThis !== 'undefined' && 'speechSynthesis' in globalThis;
    const [state, dispatch] = (0, react_1.useReducer)(reducer, {
        voices: isSynthSupported ? globalThis.speechSynthesis.getVoices() : [],
        boundary: defaultBoundary,
        isPlaying: false,
        isPaused: false,
        isMuted: false,
        isError: false,
        isReady: isSynthSupported && typeof fetchAudioData === 'undefined'
    });
    const [ttsChildren, spokenText] = (0, react_1.useMemo)(() => {
        const buffer = { text: '' };
        const parsed = parseChildrenRecursively({
            children,
            buffer,
            markColor,
            markBackgroundColor,
            markTextAsSpoken,
            boundary: state.boundary
        });
        spokenTextRef.current = buffer.text.trim();
        return [parsed, spokenTextRef.current];
    }, [children, state.boundary, markColor, markBackgroundColor, markTextAsSpoken]);
    const controller = (0, react_1.useMemo)(() => isSynthSupported
        ? new controller_js_1.Controller({
            lang,
            voice,
            fetchAudioData
        })
        : new controller_js_1.ControllerStub(), [lang, voice, isSynthSupported, fetchAudioData]);
    const play = (0, react_1.useCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        if (!isSynthSupported) {
            handleNotSupported();
        }
        else if (state.isPaused) {
            yield controller.resume();
        }
        else {
            // Replay gives a more consistent/expected experience
            yield controller.replay();
        }
        dispatch({ type: 'play' });
    }), [controller, state.isPaused, isSynthSupported, handleNotSupported]);
    const pause = (0, react_1.useCallback)(() => {
        controller.pause();
        dispatch({ type: 'pause' });
    }, [controller]);
    const replay = (0, react_1.useCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        yield controller.replay();
        dispatch({ type: 'reset' });
    }), [controller]);
    const stop = (0, react_1.useCallback)(() => {
        controller.cancel();
        dispatch({ type: 'stop' });
    }, [controller]);
    const toggleMuteHandler = (0, react_1.useCallback)((callback) => __awaiter(void 0, void 0, void 0, function* () {
        const wasMuted = parseFloat(controller.volume.toFixed(2)) === controller.volumeMin;
        if (wasMuted) {
            yield controller.unmute();
            dispatch({ type: 'unmuted' });
        }
        else {
            yield controller.mute();
            dispatch({ type: 'muted' });
        }
        if (typeof callback === 'function') {
            callback(wasMuted);
        }
    }), [controller]);
    const playOrPause = (0, react_1.useCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        if (!isSynthSupported) {
            handleNotSupported();
        }
        else if (state.isPlaying) {
            pause();
        }
        else {
            yield play();
        }
    }), [state.isPlaying, isSynthSupported, handleNotSupported, pause, play]);
    const playOrStop = (0, react_1.useCallback)(() => __awaiter(void 0, void 0, void 0, function* () {
        if (!isSynthSupported) {
            handleNotSupported();
        }
        else if (state.isPlaying) {
            stop();
        }
        else {
            yield play();
        }
    }), [state.isPlaying, isSynthSupported, handleNotSupported, stop, play]);
    const [get, set] = (0, react_1.useMemo)(() => [
        {
            lang() {
                return controller.lang;
            },
            rate() {
                return controller.rate;
            },
            pitch() {
                return controller.pitch;
            },
            volume() {
                return controller.volume;
            },
            preservesPitch() {
                return controller.preservesPitch;
            }
        },
        {
            lang(value) {
                controller.lang = value;
            },
            rate(value) {
                controller.rate = value;
            },
            pitch(value) {
                controller.pitch = value;
            },
            volume(value) {
                controller.volume = value;
            },
            preservesPitch(value) {
                controller.preservesPitch = value;
            }
        }
    ], [controller]);
    // Controller event listeners
    const onStartHandler = (0, react_1.useCallback)((evt) => {
        dispatch({ type: 'play' });
        if (typeof onStart === 'function') {
            onStart(evt.detail);
        }
    }, [onStart]);
    const onPauseHandler = (0, react_1.useCallback)((evt) => {
        if (typeof onPause === 'function') {
            onPause(evt.detail);
        }
    }, [onPause]);
    const onEndHandler = (0, react_1.useCallback)((evt) => {
        dispatch({ type: 'end' });
        if (typeof onEnd === 'function') {
            onEnd(evt.detail);
        }
    }, [onEnd]);
    const onReady = (0, react_1.useCallback)(() => {
        dispatch({ type: 'ready' });
    }, []);
    const onErrorHandler = (0, react_1.useCallback)((evt) => {
        dispatch({ type: 'error' });
        if (typeof onError === 'function') {
            onError(evt.detail);
        }
    }, [onError]);
    const onBoundaryHandler = (0, react_1.useCallback)((evt) => {
        dispatch({ type: 'boundary', payload: evt.detail.boundary });
        if (typeof onBoundary === 'function') {
            onBoundary(evt.detail.boundary, evt.detail.evt);
        }
    }, [onBoundary]);
    const onVolume = (0, react_1.useCallback)((evt) => {
        const volume = evt.detail;
        const min = controller.volumeMin;
        if (volume === min && controller.volume !== min) {
            dispatch({ type: 'muted' });
        }
        if (volume !== min && controller.volume === min) {
            dispatch({ type: 'unmuted' });
        }
        if (typeof onVolumeChange === 'function') {
            onVolumeChange(volume);
        }
    }, [onVolumeChange, controller]);
    const onPitch = (0, react_1.useCallback)((evt) => {
        if (typeof onPitchChange === 'function') {
            onPitchChange(evt.detail);
        }
    }, [onPitchChange]);
    const onRate = (0, react_1.useCallback)((evt) => {
        if (typeof onRateChange === 'function') {
            onRateChange(evt.detail);
        }
    }, [onRateChange]);
    (0, react_1.useEffect)(() => {
        controller.text = spokenText;
    }, [spokenText, controller]);
    (0, react_1.useEffect)(() => {
        if (rate && Number.isFinite(rate)) {
            controller.rate = rate;
        }
        if (volume && Number.isFinite(volume)) {
            controller.volume = volume;
        }
    }, [controller, rate, volume]);
    (0, react_1.useEffect)(() => {
        const onBeforeUnload = () => {
            controller.cancel();
        };
        const initializeListeners = () => __awaiter(void 0, void 0, void 0, function* () {
            controller.addEventListener(controller_js_1.Events.PLAYING, onStartHandler);
            controller.addEventListener(controller_js_1.Events.PAUSED, onPauseHandler);
            controller.addEventListener(controller_js_1.Events.END, onEndHandler);
            controller.addEventListener(controller_js_1.Events.ERROR, onErrorHandler);
            controller.addEventListener(controller_js_1.Events.READY, onReady);
            controller.addEventListener(controller_js_1.Events.BOUNDARY, onBoundaryHandler);
            controller.addEventListener(controller_js_1.Events.VOLUME, onVolume);
            controller.addEventListener(controller_js_1.Events.PITCH, onPitch);
            controller.addEventListener(controller_js_1.Events.RATE, onRate);
            globalThis.addEventListener('beforeunload', onBeforeUnload);
            yield controller.init();
        });
        initializeListeners().catch((err) => {
            if (err instanceof Error) {
                // eslint-disable-next-line no-console
                console.error(err.message);
            }
        });
        return () => {
            globalThis.removeEventListener('beforeunload', onBeforeUnload);
            controller.removeEventListener(controller_js_1.Events.PLAYING, onStartHandler);
            controller.removeEventListener(controller_js_1.Events.PAUSED, onPauseHandler);
            controller.removeEventListener(controller_js_1.Events.END, onEndHandler);
            controller.removeEventListener(controller_js_1.Events.ERROR, onErrorHandler);
            controller.removeEventListener(controller_js_1.Events.READY, onReady);
            controller.removeEventListener(controller_js_1.Events.BOUNDARY, onBoundaryHandler);
            controller.removeEventListener(controller_js_1.Events.VOLUME, onVolume);
            controller.removeEventListener(controller_js_1.Events.PITCH, onPitch);
            controller.removeEventListener(controller_js_1.Events.RATE, onRate);
        };
    }, [
        onStartHandler,
        onBoundaryHandler,
        onPauseHandler,
        onEndHandler,
        onReady,
        onErrorHandler,
        onBoundary,
        onVolume,
        onPitch,
        onRate,
        controller
    ]);
    (0, react_1.useEffect)(() => {
        const handleAutoPlay = () => __awaiter(void 0, void 0, void 0, function* () {
            if (autoPlay) {
                if (!isSynthSupported) {
                    handleNotSupported();
                }
                else if (state.isReady) {
                    yield controller.replay();
                    dispatch({ type: 'play' });
                }
            }
        });
        handleAutoPlay().catch((err) => {
            if (err instanceof Error) {
                // eslint-disable-next-line no-console
                console.error(`Unable to auto play: ${err.message}`);
            }
        });
    }, [autoPlay, controller, state.isReady, isSynthSupported, handleNotSupported]);
    (0, react_1.useEffect)(() => {
        var _a;
        const onVoicesChanged = () => {
            dispatch({ type: 'voices', payload: globalThis.speechSynthesis.getVoices() });
        };
        if (typeof ((_a = globalThis.speechSynthesis) === null || _a === void 0 ? void 0 : _a.addEventListener) === 'function') {
            globalThis.speechSynthesis.addEventListener('voiceschanged', onVoicesChanged);
        }
        return () => {
            var _a;
            if (typeof ((_a = globalThis.speechSynthesis) === null || _a === void 0 ? void 0 : _a.removeEventListener) === 'function') {
                globalThis.speechSynthesis.removeEventListener('voiceschanged', onVoicesChanged);
            }
        };
    }, []);
    return {
        get,
        set,
        state,
        spokenText,
        ttsChildren,
        isSynthSupported,
        play,
        stop,
        pause,
        replay,
        playOrStop,
        playOrPause,
        toggleMute: toggleMuteHandler
    };
};
exports.useTts = useTts;
